function vector=djgenchirp(sweep_khz,duration_ms,smooth_percent,direction,smooth_sine,nb_pt,shape_path,version,psyche)

% if psyche=0 : single chirp
% if psyche==1 : two successive chirps running in opposite directions
% if psyche==2 : chirp and back chirp summed (two saltire chirps). Need to double the power.


% Give titles to the shapes which will be generated. The titles will be
% specific to the type of chirp and will contain information about
% parameters used to generate each shape. The title will be stored in the
% header of the Bruker file which will be generated.
if psyche==0
    title=['chirp' ];
end
    title=['chirp'  ];

if psyche==1
    title=['djpsyche_single '  ];
end

if psyche==2
    title=['djpsyche_double '  ];
end

if psyche==0
    title=[title  num2str(sweep_khz) ' kHz ' num2str(duration_ms) ' ms ' num2str( nb_pt) ' pt smooth: ' num2str(smooth_percent) ' direction ' num2str(direction) ];
else
    title=[title  num2str(sweep_khz) ' kHz 2x' num2str(duration_ms) ' ms ' num2str( nb_pt) ' pt smooth: ' num2str(smooth_percent) ' direction ' num2str(direction) ];    
end

% Prelocate the shape (array equal to the number of points defining the shape)
out=ones(1,nb_pt);

% Calculate the number of points which will represent the smoothed part of the shape
smooth_part=round(nb_pt*0.01*smooth_percent);

% Perform the smoothing (the type of smoothing depends on the option "smooth_sine")
% The smoothing is performed by using the sine function running from 0 to pi/2
if smooth_sine % this is for sin squared smoothing
    out(1,1:smooth_part)=out(1,1:smooth_part)               .*      (0.5+0.5*sin(  -pi/2:pi/smooth_part:pi/2-pi/smooth_part ));
    out(1,end-smooth_part+1:end)=out(1,end-smooth_part+1:end).*fliplr(0.5+0.5*sin(  -pi/2:pi/smooth_part:pi/2-pi/smooth_part ));

else %this is for sin smoothing
    out(1,1:smooth_part)        =out(1,1:smooth_part)               .*sin(  0:0.5*pi/smooth_part:pi/2-0.5*pi/smooth_part );
    out(1,end-smooth_part+1:end)=out(1,end-smooth_part+1:end).*fliplr(sin(  0:0.5*pi/smooth_part:pi/2-0.5*pi/smooth_part ));
end

% Scale up to 100 (for Bruker shape files)
out=100*out;

% Sort out the units
sweep_hz=sweep_khz*1000;
duration_us=1000*(duration_ms);

% Calculate the maximum phase [degrees]. Note the phase quadratic time dependence, hence sqrt in the equation
max_angle=sqrt(360*duration_us*sweep_hz/2e6);

% Calculate the frequency range over which the chirp works and normalize it
frq_range=-max_angle/2+0.5*(max_angle/(nb_pt)):max_angle/(nb_pt):max_angle/2-0.5*(max_angle/(nb_pt));
frq_range2=frq_range.*frq_range/360; % normalized

% Parabolic phase
table_angle=frq_range2*direction*360;

% Phase-modulated chirp
outc1=out.*exp( j*2*pi.*frq_range2*direction);

% Phase-modulated chirp in opposite direction
outc2=out.*exp(-j*2*pi.*frq_range2*direction);

% Saltire chirp
outc=outc1*0.5+0.5*outc2;

% % Plot the chirp
% clf
% plot(real(out),'g-');
% hold on
% plot(real(outc),'g:');
% plot(imag(out),'r-');
% plot(imag(outc),'r:');

% Plot the chirp
figure(10)
plot(real(out),'g-');
figure(11)
plot(real(outc),'g:');
figure(12)
plot(imag(out),'r-');
figure(13)
plot(imag(outc),'r:');

% Take care of the number of points which must be doubled in case of double-chirp element.
nb_toto_pt=size(out,2);

% Give names to the files which will be generated.
if psyche==0
    name=[shape_path '/djchirp.'  num2str(sweep_khz) 'kHz' num2str(duration_ms) 'm.' num2str( nb_toto_pt) '' ];
end
if psyche==1
    nb_toto_pt=nb_toto_pt*2;
    name=[shape_path '/djpsyche_single.'  num2str(sweep_khz) 'kHz_2x' num2str(duration_ms) 'm.' num2str( nb_toto_pt) '' ];
end
if psyche==2
    nb_toto_pt=nb_toto_pt*2;
    name=[shape_path '/djpsyche_double'  num2str(sweep_khz) 'kHz_2x' num2str(duration_ms) 'm.' num2str( nb_toto_pt) '' ];
end

% Open file and prepeare its header consistent with Bruker format
fid = fopen(name,'w');
fprintf(fid,'##TITLE= %s\n',title);
fprintf(fid,'##JCAMP-DX= 5.00 Bruker JCAMP library\n');
fprintf(fid,'##DATA TYPE= Shape Data\n');
fprintf(fid,['##ORIGIN= Matlab program generating shapes Version ' version '\n']);
fprintf(fid,'##OWNER= <nmr>\n');
fprintf(fid,'##DATE= 2015/00/06\n');
fprintf(fid,'##TIME= 00:00:00\n');
fprintf(fid,'##$SHAPE_PARAMETERS= Type: matlab generation\n');

% Minimum amplitude of the chirp
tmp=min(min(abs(out(1,:))));
fprintf(fid,'##MINX= %.6E\n',tmp);

% Maximum aplitude of the chirp
tmp=max(max(abs(out(1,:))));
fprintf(fid,'##MAXX= %.6E\n',tmp);

% Minimum phase of the chirp
tmp_table_angle=angle(out(1,:))*360/pi;
tmp_table_angle=tmp_table_angle+360*(tmp_table_angle<0);
tmp=min(min(tmp_table_angle));
fprintf(fid,'##MINY= %.6E\n',tmp);

% Maximum phase of the chirp
tmp=max(max(tmp_table_angle));
fprintf(fid,'##MAXY= %.6E\n',tmp);

fprintf(fid,'##$SHAPE_EXMODE= Adiabatic\n');
fprintf(fid,'##$SHAPE_TOTROT= 1.800000E02\n');
fprintf(fid,'##$SHAPE_TYPE= Inversion\n');
fprintf(fid,'##$SHAPE_USER_DEF= \n');
fprintf(fid,'##$SHAPE_REPHFAC= \n');
fprintf(fid,'##$SHAPE_BWFAC= 0\n');
fprintf(fid,'##$SHAPE_BWFAC50= \n');
fprintf(fid,'##$SHAPE_INTEGFAC= 0\n');
fprintf(fid,'##$SHAPE_MODE= %d\n',0);
fprintf(fid,'##NPOINTS= %d\n',nb_toto_pt);
fprintf(fid,'##XYPOINTS= (XY..XY)\n');

% Print value of amplitude (left column) and phase (right column) for each
% point of the phase-modulated shape
if psyche<2
    for loop=1:size(out,2)
        fprintf(fid,'%.6E, %.6E\n',abs(out(loop)),mod(table_angle(loop),360)); % normal chirp
    end
    if psyche>0
        for loop=1:size(out,2)
            fprintf(fid,'%.6E, %.6E\n',abs(out(loop)),mod(-table_angle(loop),360)); % reversed-sweep chirp (following the first)
        end
    end
    
else
    
% Print value of amplitude (left column) and phase (right column) for each
% point of the saltire shape
    for loop=1:size(out,2)
        if (angle(out(loop))*180/pi) < 0
            fprintf(fid,'%.6E, %.6E\n',abs(outc(loop)),360+angle(outc(loop))*180/pi);
        else
            fprintf(fid,'%.6E, %.6E\n',abs(outc(loop)),angle(outc(loop))*180/pi);
        end
    end
    
    if psyche>0
        for loop=1:size(out,2)
            if (angle(out(loop))*180/pi) < 0
                fprintf(fid,'%.6E, %.6E\n',abs(outc(loop)),360+angle(outc(loop))*180/pi);
            else
                fprintf(fid,'%.6E, %.6E\n',abs(outc(loop)),angle(outc(loop))*180/pi);
            end
        end
    end
end

fprintf(fid,'##END=\n');
fclose(fid);
vector=out;
return