clear all
close all

%%% Prepare the basic shape which need to be modulated
% Specify the path where the basic shapes are (often, Topspin)
path='/opt/topspin3.2/exp/stan/nmr/';% Specify the path up to the "lists" folder
shape_path=[path 'lists/wave/user/'];% Location of the shaped pulses

% Choose 0 or 1 if you want to display the shape in a figure 
display_figure=1;

% Specify the exact name of the basic shape
shape_name='RSnob.29000'; % The following topspin command should generate the shape automatically : "st generate RSnob 29000 filename=rsnob.29000"
prename='rsnob';

% Specify how many BLOCKS of irradiation sites should be generated. For 1H typically two blocks 
% and for 13C six blocks are used to ensure homogeneous excitation over a chosen SW ('true_full_width' below)
% For more details check: DOI: 10.1002/anie.201500831 and the in press
% paper on 13C decoupling
how_many_main_chunks_in_the_full_spectrum=1; % Typically 1 for 1H and 5 for 13C
nb_blocks=how_many_main_chunks_in_the_full_spectrum+1; % Results 2 for 1H and 6 for 13C

% Specify the SW that will be observed and should be homogenously covered
true_full_width=10; % typical for 1H

% Spectral domain which will be covered in each block
sw_full=true_full_width/(how_many_main_chunks_in_the_full_spectrum);

% Choose 0 or 1 if you want to display the positions of irradiations 
fig=1;

% List here the Larmor frequency of the spectrometer (remember to adjust for a given isotope)
for larmor=[ 500 ] 
    
% Specify how many irradiation sites should be generated in 1 block of irradiation sites.
% Warning. Nothing limits you here. If you use too many modulations and too short pulses the necessary power may be too high.
    for nb_mod=[50    ]
        
% Specify here if you want non-equidistant modulation [1], equidistant
% modulation [0] or both [0 1]
        for non_equ_dist=[0 1]
            
% List here the durations of the pulses you want to use
            duration_ms_table=[100 120];
                      
% Calculate positions of the irradiation sites 
            list=zeros(1,nb_blocks*nb_mod);
            full_hz=larmor*sw_full;
            if non_equ_dist
                step_hz=(2/3)*(full_hz/(nb_mod));
            else
                step_hz=(full_hz/nb_mod);
            end
            delta_delta=non_equ_dist*step_hz/(1*(nb_mod-1));
            
            cur=0;
            for loop_pos=1:nb_mod
                for lnb=0:how_many_main_chunks_in_the_full_spectrum
                    list(1,loop_pos+nb_mod*lnb)=cur+full_hz*lnb;
                end
                cur=cur+step_hz+(loop_pos-1)*delta_delta;
            end
            list=list-0.5*(max(max(list)));
            
% Read the basic shape to be modulated from the Bruker format   
            vector=read_bruk([shape_path shape_name],display_figure);
            if size(vector,2)==0
                error(['Could not read file : ' shape_path shape_name])
            end

% Prepare number for figure 
              if fig
                       figure(1000);clf;hold on
              end
            
% Generate the modulated shape

            for duration_ms=duration_ms_table  
                comment=['delta= ' num2str(step_hz,'%.3f') ' delta_delta=' num2str(delta_delta,'%.3f')];
                max_vector=max(max(abs(vector)));
                out=vector*0;
                pt=size(vector,2);
                ramp=(1/1)*(-0.5+0.5/pt:1/pt:0.5-0.5/pt);% -0.5 to 0.5
                normil=size(list,2);
                for loop_freq=1:size(list,2)
                    
                     if non_equ_dist
                     sup_nam='nem';
                     else
                     sup_nam='equ';
                     end
                
                     if fig
                     plot(list(1,loop_freq),0,'x');
                     text(list(1,loop_freq),1,num2str(loop_freq));
                     for_title=[ num2str(larmor) '_MHz_' sup_nam '_' prename  '_'  num2str(nb_blocks) 'x' num2str(nb_mod) '_'  num2str(duration_ms)  '_ms_'];
                     title(for_title,'interpreter','none');
                     end
                   
                     phase_shift=exp((-2*pi*j*ramp).*(duration_ms/1000*(list(1,loop_freq))));
                     out=out+vector.*phase_shift/normil;
                     tti=list(1,loop_freq);
                end
               
                
                % Save the modulated file in a chosen directory (often in Topspin, often
                % where the basic shape was taken from). The name of the file depends on
                % parameters specified above. 

                file_name=[shape_path num2str(larmor) '_MHz_' sup_nam '_' prename  '_'  num2str(nb_blocks) 'x' num2str(nb_mod) '_'  num2str(duration_ms)  '_ms' ];
                write_bruk(file_name,comment,out,nb_blocks*nb_mod);
                disp(['Writing Bruker wave file: ' shape_path num2str(larmor) '_MHz_' sup_nam '_' prename  '_'  num2str(nb_blocks) 'x' num2str(nb_mod) '_'  num2str(duration_ms)  '_ms']);
                display_ft(file_name,duration_ms,larmor);

            end
        end
    end
end
disp(shape_path)

